package com.bjlt.spider.htmlunit;

import com.gargoylesoftware.htmlunit.CookieManager;
import com.gargoylesoftware.htmlunit.httpclient.HtmlUnitCookieStore;
import com.gargoylesoftware.htmlunit.util.Cookie;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.junit.Before;
import org.junit.Test;



/**
 * auth : luyanan0718@163.com
 * com.bjlt.spider.htmlunit
 * CreateTime:2017/10/27-9:33
 * introduce:
 **/

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.junit.Before;
import org.junit.Test;

import com.gargoylesoftware.htmlunit.httpclient.HtmlUnitCookieStore;
import com.gargoylesoftware.htmlunit.util.Cookie;

public class HtmlUnitCookieStoreTest {


    private CookieManager mgr_;
    private HtmlUnitCookieStore store_;

    /**
     * Setup.
     */
    @Before
    public void setUp() {
        mgr_= new CookieManager();
      store_  = new HtmlUnitCookieStore(mgr_);
    }

    /**
     * Simple test for addCookie.
     */
    @Test
    public void addCookie() {
        final BasicClientCookie c = new BasicClientCookie("myname", "myvalue");
        c.setDomain("localhost");
        store_.addCookie(c);

        System.out.println( mgr_.getCookies());
        assertEquals(1, mgr_.getCookies().size());
        assertEquals("myvalue", mgr_.getCookie("myname").getValue());
    }

    /**
     * Simple test for getCookies.
     */
    @Test
    public void getCookies() {
        mgr_.addCookie(new Cookie("localhost", "myname", "myvalue"));
        mgr_.addCookie(new Cookie("localhost", "myname2", "myvalue2"));

        final List<org.apache.http.cookie.Cookie> cookies = store_.getCookies();
        assertEquals(2, cookies.size());
        assertTrue(cookies.contains(new MyCookie("myname", "myvalue")));
        assertTrue(cookies.contains(new MyCookie("myname2", "myvalue2")));

        mgr_.setCookiesEnabled(false);
        assertTrue(store_.getCookies().isEmpty());
    }

    /**
     * Simple test for clearExpired.
     */
    @Test
    public void clearExpired() {
        mgr_.addCookie(new Cookie("localhost", "myname", "myvalue"));
        final Cookie cookie = new Cookie("localhost", "myname2", "myvalue2", null,
                new Date(System.currentTimeMillis() + 10_000), false);
        mgr_.addCookie(cookie);

        assertTrue(store_.clearExpired(new Date(System.currentTimeMillis() + 20_000)));
        assertFalse(store_.clearExpired(new Date(System.currentTimeMillis() + 20_000)));

        final List<org.apache.http.cookie.Cookie> cookies = store_.getCookies();
        assertEquals(1, cookies.size());
        assertTrue(cookies.contains(new MyCookie("myname", "myvalue")));
    }

    /**
     * Simple test for clear.
     */
    @Test
    public void clear() {
        mgr_.addCookie(new Cookie("localhost", "myname", "myvalue"));
        mgr_.addCookie(new Cookie("localhost", "myname2", "myvalue2"));

        assertFalse(store_.getCookies().isEmpty());

        store_.clear();
        assertTrue(store_.getCookies().isEmpty());
    }

    private static final class MyCookie extends BasicClientCookie {
        MyCookie(final String name, final String value) {
            super(name, value);
        }

        @Override
        public boolean equals(final Object obj) {
            return obj instanceof org.apache.http.cookie.Cookie
                    && new EqualsBuilder()
                    .append(getName(), ((org.apache.http.cookie.Cookie) obj).getName())
                    .append(getValue(), ((org.apache.http.cookie.Cookie) obj).getValue())
                    .isEquals();
        }

        @Override
        public int hashCode() {
            return super.hashCode();
        }
    }
}
